% Adjusted from Malec, L., Skacel, F., Trujillo-Ortiz, A. (2007). FA: Factor analysis by principal axis factoring. MATLAB file. Retrieved from https://www.mathworks.com/matlabcentral/fileexchange/14115-fa

function [Lobl,c,Fscores,Fscores_T,Rreduced,phi] = FA(X,s,TL,R)
[~,k]=size(X);    % k = number of variables
X=zscore(X);

warning off all;c = ones(k,1) - 1 ./ diag(mldivide(R,eye(k)));warning on all % Communality estimation by coefficients of multiple correlation.
Rreduced=R-diag(diag(R))+diag(c);

% Iteration cycle which maximizes factors correlations.
for k = 1:9999
    [U,D] = svd(R - diag(diag(R)) + diag(c),0);
    N = U * sqrt(D(:,1:s));
    p = c;
    c = sum(N.^2,2);
    c(c>1)=1; % Heywood case in communality estimates
    if max(abs(c - p))<10^-3
        break
    end
end

% First perform a Varimax rotation to have a simple structure

% Normalization of factor loadings.
h = sqrt(c);
N = N ./ repmat(h,1,s);

% Initial choice of eigenvalues and loadings labelling.
L = N;
z = 0;

% Iteration cycle maximizing variance of individual columns.
for l = 1:35
    [A,S,M] = svd(N' * (n * L.^3 - L * diag(sum(L.^2))),0);
    L = N * A * M';
    b = z;
    z = sum(diag(S));
    if abs(z - b)<.00001
        break
    end
end
L=L.*repmat(h,1,s); % Unnormalization of factor loadings.
L=ordering(L,TL); % Recover the order and sign of the factors

[L,phi]=GPFoblq(L,eye(s)); % Oblimin rotation

temp=find(c<1);
sqrtPsi = sqrt(1-c(temp));
invsqrtPsi = diag(1./sqrtPsi);
Fscores = (X(:,temp)*invsqrtPsi)/(L(temp,:)'*invsqrtPsi);

sqrtPsi_T = sqrt(1-sum(TL'.^2,1));
invsqrtPsi_T = diag(1./sqrtPsi_T);
Fscores_T = (X*invsqrtPsi_T)/(TL'*invsqrtPsi_T);  % Factor scores if we had retrieved the true loadings

Lobl=L;