% De Winter, J. C. F., Dodou, D., & Wieringa, P. A. (2009). Exploratory factor analysis with small sample sizes. Multivariate Behavioral Research, 44, 147-181.

clear all;close all;clc;warning on all %#ok<CLALL>
CB=combvec([.4 .9],1:8,floor(logspace(log10(10),log10(200),20)),[25 100 1000])'; % 1. loading, 2. number of factors, 3. number of variables, 4. sample size
[CC,FSC,HW]=deal(NaN(size(CB,1),5000));

for k=1:size(CB,1)
    lambda = CB(k,1); m = CB(k,2);p = CB(k,3);N=CB(k,4);
    CB(k,:)
    if N==25 && m>2
        REPS=500;
    elseif N==25 && m==2
        REPS=1000;
    elseif N==25 && m==1
        REPS=5000;
    end
    if N==100 && m>2
        REPS=200;
    elseif N==100 && m==2
        REPS=400;
    elseif N==100 && m==1
        REPS=2000;
    end
    if N==1000 && m>2
        REPS=50;
    elseif N==1000 && m==2
        REPS=100;
    elseif N==1000 && m==1
        REPS=500;
    end
    if m*2<=p   % only perform  FA when the number of factors is smaller than the number of variables and when the number of variables is at least twice the number of factors
        TL=zeros(p,m);
        FD = round(1:p/m:p+1);  % equal as possible distribution of loading variables per factor
        for i=1:length(FD)-1
            TL(FD(i):FD(i+1)-1,i)=lambda; % true factor loading matrix
        end
        F=Hong_fast(m,p,N,TL,0,0,1,0,0);
        
        for j=1:REPS
            try
                X=randn(N,p);
                X=X*F';
                R=corrcoef(X); % generate observation matrix using Hong's method
                [Lobl,c,Fscores,Fscores_T] =FA(X,m,TL,R); % perform PAF, extracting m factors
                CC(k,j)=cc(TL,Lobl);                      % Tucker's congruence coefficient
                FSC(k,j)=mean(diag(corr(Fscores,Fscores_T)));
            catch error
            end
        end
    end
end
%%
REPS=10000;
[EF,EFplus1]=deal(NaN(size(CB,1),REPS));
for k=1:size(CB,1)
    lambda = CB(k,1); m = CB(k,2);p = CB(k,3);N=CB(k,4);
    CB(k,:)
    if m*2<=p  % only perform FA when the number of variables is at least twice the number of factors
        TL=zeros(p,m);
        FD = round(1:p/m:p+1);  % equal as possible distribution of loading variables per factor
        for i=1:length(FD)-1
            TL(FD(i):FD(i+1)-1,i)=lambda; % true factor loading matrix
        end
        F=Hong_fast(m,p,N,TL,0,0,1,0,0);
        for j=1:REPS
            X=randn(N,p);
            X=X*F';
            R=corrcoef(X);eigR=sort(eig(R)); % generate observation matrix using Hong's method
            EF(k,j)=eigR(p-m+1);
            EFplus1(k,j)=eigR(p-m);
        end
    end
end

%%
[ES,MeanK,MeanFSC]=deal(NaN(1,size(CB,1)));
for k=1:size(CB,1)
    V1=EF(k,:)';
    V2=EFplus1(k,:)';
    ES(k)=abs(cohend(real(V1),real(V2)));
    MeanK(k)=nanmean(CC(k,:));
    MeanFSC(k)=nanmean(FSC(k,:));
end

lambdas=unique(CB(:,1));fs=unique(CB(:,2));ps=unique(CB(:,3));Ns=unique(CB(:,4));
[ESt,MeanKt,MeanFSCt]=deal(NaN(length(lambdas),length(fs),length(ps),length(Ns)));

for i1=1:length(lambdas)
    for i2=1:length(fs)
        for i3=1:length(ps)
            for i4=1:length(Ns)
                index=find(CB(:,1)==lambdas(i1) & CB(:,2)==fs(i2) & CB(:,3)==ps(i3) & CB(:,4)==Ns(i4));
                ESt(i1,i2,i3,i4)=ES(index);
                MeanKt(i1,i2,i3,i4)=MeanK(index);
                MeanFSCt(i1,i2,i3,i4)=MeanFSC(index);
            end
        end
    end
end
%
close all

for i4=1:length(lambdas)
    ES1=log10(squeeze(ESt(i4,:,:,1)));
    ES2=log10(squeeze(ESt(i4,:,:,2)));
    ES3=log10(squeeze(ESt(i4,:,:,3)));
    MeanFSC1=-log10(1-(squeeze(MeanFSCt(i4,:,:,1))));
    MeanFSC2=-log10(1-(squeeze(MeanFSCt(i4,:,:,2))));
    MeanFSC3=-log10(1-(squeeze(MeanFSCt(i4,:,:,3))));
    MeanK1=-log10(1-(squeeze(MeanKt(i4,:,:,1))));
    MeanK2=-log10(1-(squeeze(MeanKt(i4,:,:,2))));
    MeanK3=-log10(1-(squeeze(MeanKt(i4,:,:,3))));
    
    figure;hold on
    surf(ES3,'FaceAlpha',.5,'Marker','v','FaceColor',[1.0 1.0 1.0],'MarkerFaceColor' ,'k','LineStyle','-','EdgeColor',[.5 .5 .5],'LineWidth',1,'EdgeAlpha',1);
    surf(ES2,'FaceAlpha',.5,'Marker','o','FaceColor',[.5 .5 .5],'MarkerFaceColor' ,'w','LineStyle','--','EdgeColor',[.5 .5 .5],'LineWidth',1,'EdgeAlpha',1);
    surf(ES1,'FaceAlpha',1, 'Marker','s','FaceColor',[.0 .0 .0],'MarkerFaceColor' ,[.5 .5 .5],'EdgeColor',[.3 .3 .3])
    view(254,5)
    xlabel('p','FontSize',20);ylabel('f','FontSize',20);zlabel('ES','FontSize',20)
    colormap(bone)
    ztick=[1 2 3 4 5 6 7 8 9 10 20 30 40 50 60 70 80 90 100 200];
    set(gca,'YTick',fs,'XTick',round(linspace(1,length(ps),8)),'XTickLabel',ps(round(linspace(1,length(ps),8))),'ZTick',log10(ztick),'ZTickLabel',ztick)
    h = findobj(gcf,'FontName','Helvetica');set(h,'FontSize',12)
    
    figure;hold on
    surf(MeanFSC3,'facealpha',.5,'Marker','v','FaceColor',[1.0 1.0 1.0],'MarkerFaceColor' ,'k','LineStyle','-','EdgeColor',[.5 .5 .5],'LineWidth',1,'EdgeAlpha',1);
    surf(MeanFSC2,'facealpha',.5,'Marker','o','FaceColor',[.5 .5 .5],'MarkerFaceColor' ,'w','LineStyle','--','EdgeColor',[.5 .5 .5],'LineWidth',1,'EdgeAlpha',1);
    surf(MeanFSC1,'facealpha',1, 'Marker','s','FaceColor',[.0 .0 .0],'MarkerFaceColor',[.5 .5 .5],'EdgeColor',[.3 .3 .3])
    view(254,5)
    xlabel('p','FontSize',20);ylabel('f', 'FontSize',20);zlabel('mean FSC', 'FontSize',20)
    colormap(bone)
    set(gca,'YTick',fs,'XTick',round(linspace(1,length(ps),8)),'XTickLabel',ps(round(linspace(1,length(ps),8))),'ZTick',-log10(1-[.1 .2 .3 .4 .5 .6 .7 .8 .9 .99 .999 .9999 .99999 .999999]),'ZTickLabel',[.1 .2 .3 .4 .5 .6 .7 .8 .9 .99 .999 .9999 .99999 .999999])
    h = findobj(gcf,'FontName','Helvetica');set(h,'FontSize',12)
    
    figure;hold on
    surf(MeanK3,'facealpha',.5,'Marker','v','FaceColor',[1.0 1.0 1.0],'MarkerFaceColor' ,'k','LineStyle','-','EdgeColor',[.5 .5 .5],'LineWidth',1,'EdgeAlpha',1);
    surf(MeanK2,'facealpha',.5,'Marker','o','FaceColor',[.5 .5 .5],'MarkerFaceColor' ,'w','LineStyle','--','EdgeColor',[.5 .5 .5],'LineWidth',1,'EdgeAlpha',1);
    surf(MeanK1,'facealpha',1, 'Marker','s','FaceColor',[.0 .0 .0],'MarkerFaceColor',[.5 .5 .5],'EdgeColor',[.3 .3 .3])
    view(254,5)
    xlabel('p','FontSize',20);ylabel('f','FontSize',20);zlabel('mean K','FontSize',20)
    colormap(bone)
    ztick=[1 2 3 4 5 6 7 8 9 10 20 30 40 50 60 70 80 90 100 200];
    set(gca,'YTick',fs,'XTick',round(linspace(1,length(ps),8)),'XTickLabel',ps(round(linspace(1,length(ps),8))),'ZTick',-log10(1-[.1 .2 .3 .4 .5 .6 .7 .8 .9 .99 .999 .9999 .99999 .999999]),'ZTickLabel',[.1 .2 .3 .4 .5 .6 .7 .8 .9 .99 .999 .9999 .99999 .999999])
    h = findobj(gcf,'FontName','Helvetica');set(h,'FontSize',12)
end
