% De Winter, J. C. F., Dodou, D., & Wieringa, P. A. (2009). Exploratory factor analysis with small sample sizes. Multivariate Behavioral Research, 44, 147-181.

clear all;close all;clc;warning off all %#ok<CLALL>
reps=999999;
N=17;
CASES=1:57;
[CC,CC1,CC2,CC3,FSC,FSC1,FSC2,FSC3,EF,EFplus1,HW,EFreduced,EFreducedplus1]=deal(NaN(reps,max(CASES)));

for cases=CASES
    disp(cases)
    e=0; % model error
    f=0; % interfactor correlation
    oblique=1; % oblimin
    ifcw=1;
    eps=.1;
    q=200;
    TL= [repmat([.8 0 0],8,1);repmat([0 .8 0],8,1);repmat([0 0 .8],8,1)];TL2=TL;
    switch cases
        case 2           %% 1. IFC two factors
            ifcw=2;f=0.1;
        case 3
            ifcw=2;f=0.3;
        case 4
            ifcw=2;f=0.5;
        case 5
            ifcw=2;f=0.7;
        case 6        %% 2. IFC all factors
            f=0.1;
        case 7
            f=0.3;
        case 8
            f=0.5;
        case 9
            f=0.7;
        case 10      %% 3. model error
            e=.05;
        case 11
            e=.10;
        case 12
            e=.15;
        case 13
            e=.2;
        case 14      %% 4. model error  epsilon .15
            e=.2;eps=.05;
        case 15
            e=.2;eps=.15;
        case 16
            e=.2;eps=.25;
        case 17
            e=.2;eps=.35;
        case 18     %% 5. weak loaders append
            TL= [repmat([.8 0 0],8,1);repmat([0 .8 0],8,1);repmat([0 0 .8],8,1);repmat([.6 0 0],2,1);repmat([0 .6 0],2,1);repmat([0 0 .6],2,1)];
        case 19
            TL= [repmat([.8 0 0],8,1);repmat([0 .8 0],8,1);repmat([0 0 .8],8,1);repmat([.6 0 0],4,1);repmat([0 .6 0],4,1);repmat([0 0 .6],4,1)];
        case 20
            TL= [repmat([.8 0 0],8,1);repmat([0 .8 0],8,1);repmat([0 0 .8],8,1);repmat([.6 0 0],8,1);repmat([0 .6 0],8,1);repmat([0 0 .6],8,1)];
        case 21
            TL= [repmat([.8 0 0],8,1);repmat([0 .8 0],8,1);repmat([0 0 .8],8,1);repmat([.6 0 0],32,1);repmat([0 .6 0],32,1);repmat([0 0 .6],32,1)];
        case 22     %% 6. weak loaders replace
            TL= [repmat([.8 0 0],7,1);repmat([0 .8 0],7,1);repmat([0 0 .8],7,1);repmat([.6 0 0],1,1);repmat([0 .6 0],1,1);repmat([0 0 .6],1,1);];
        case 23
            TL= [repmat([.8 0 0],6,1);repmat([0 .8 0],6,1);repmat([0 0 .8],6,1);repmat([.6 0 0],2,1);repmat([0 .6 0],2,1);repmat([0 0 .6],2,1);];
        case 24
            TL= [repmat([.8 0 0],4,1);repmat([0 .8 0],4,1);repmat([0 0 .8],4,1);repmat([.6 0 0],4,1);repmat([0 .6 0],4,1);repmat([0 0 .6],4,1);];
        case 25
            TL= [repmat([.8 0 0],2,1);repmat([0 .8 0],2,1);repmat([0 0 .8],2,1);repmat([.6 0 0],6,1);repmat([0 .6 0],6,1);repmat([0 0 .6],6,1);];
        case 26     %% 7. simply remove variables. It is hypothesized that we do not see a discontinuity when p exceeds N
            TL= [repmat([.8 0 0],4,1);repmat([0 .8 0],4,1);repmat([0 0 .8],4,1)];  % p=12<N
        case 27
            TL= [repmat([.8 0 0],5,1);repmat([0 .8 0],5,1);repmat([0 0 .8],5,1)];  % p=15<N
        case 28
            TL= [repmat([.8 0 0],6,1);repmat([0 .8 0],6,1);repmat([0 0 .8],6,1)];  % p=18>N
        case 29
            TL= [repmat([.8 0 0],16,1);repmat([0 .8 0],16,1);repmat([0 0 .8],16,1)];  % p48>N
        case 30    %% 8. negative loadings
            TL= [repmat([.8 0 0],8,1);repmat([0 .8 0],8,1);repmat([0 0 -.8],8,1)];  % negative loadings 1 factor
        case 31
            TL= [repmat([.8 0 0],8,1);repmat([0 .8 0],8,1);repmat([0 0 .8;0 0 -.8],4,1)];  % negative loadings within 1 factor
        case 32
            temp=find(TL);TL(temp(1:2:end))=.8;TL(temp(2:2:end))=-.8;  % negative loadings within all factors
        case 33    % random sign of all loadings
        case 34    % 9. unequal loadings of factors, keeping communalities .64
            TL(TL(:,1)>0,1)=.85;TL(TL(:,3)>0,3)=sqrt(0.8^2-(0.85^2-0.8^2));
        case 35
            TL(TL(:,1)>0,1)=.9;TL(TL(:,3)>0,3)=sqrt(0.8^2-(0.9^2-0.8^2));
        case 36
            TL(TL(:,1)>0,1)=.95;TL(TL(:,3)>0,3)=sqrt(0.8^2-(0.95^2-0.8^2));
        case 37
            TL(TL(:,1)>0,1)=.99;TL(TL(:,3)>0,3)=sqrt(0.8^2-(0.99^2-0.8^2));
        case 38  % 10. unequal loadings withing factors
            temp=find(TL);TL(temp(1:2:end))=.85;TL(temp(2:2:end))=sqrt(0.8^2-(0.85^2-0.8^2));
        case 39
            temp=find(TL);TL(temp(1:2:end))=.9;TL(temp(2:2:end))=sqrt(0.8^2-(0.9^2-0.8^2));
        case 40
            temp=find(TL);TL(temp(1:2:end))=.95;TL(temp(2:2:end))=sqrt(0.8^2-(0.95^2-0.8^2));
        case 41
            temp=find(TL);TL(temp(1:2:end))=.99;TL(temp(2:2:end))=sqrt(0.8^2-(0.99^2-0.8^2));
        case 42  % 11. crossloadings
            TL(1,:)=[.8 .2 0 ];
            TL(2,:)=[.8 -.2 0 ];
        case 43
            TL(1,:)=[.8 .4 0 ];
            TL(2,:)=[.8 -.4 0 ];
        case 44
            TL(1,:)=[.8 .2 0 ];
            TL(2,:)=[.8 -.2 0 ];
            TL(3,:)=[.8 .2 0 ];
            TL(4,:)=[.8 -.2 0 ];
        case 45
            TL(1,:)=[.8 .4 0 ];
            TL(2,:)=[.8 -.4 0 ];
            TL(3,:)=[.8 .4 0 ];
            TL(4,:)=[.8 -.4 0 ];
        case 50  % 12. unequal p/f. one weak factor
            TL= [repmat([.8 0 0],8,1);repmat([0 .8 0],8,1);repmat([0 0 .8],6,1)];
        case 51
            TL= [repmat([.8 0 0],8,1);repmat([0 .8 0],8,1);repmat([0 0 .8],4,1)];
        case 52
            TL= [repmat([.8 0 0],8,1);repmat([0 .8 0],8,1);repmat([0 0 .8],3,1)];
        case 53
            TL= [repmat([.8 0 0],8,1);repmat([0 .8 0],8,1);repmat([0 0 .8],2,1)];
        case 54  % 13. unequal p/f. two weak factors
            TL= [repmat([.8 0 0],8,1);repmat([0 .8 0],6,1);repmat([0 0 .8],6,1)];
        case 55
            TL= [repmat([.8 0 0],8,1);repmat([0 .8 0],4,1);repmat([0 0 .8],4,1)];
        case 56
            TL= [repmat([.8 0 0],8,1);repmat([0 .8 0],3,1);repmat([0 0 .8],3,1)];
        case 57
            TL= [repmat([.8 0 0],8,1);repmat([0 .8 0],2,1);repmat([0 0 .8],2,1)];
    end
    m=size(TL,2);p=size(TL,1);
    
    for REP=1:reps
        switch cases
            case 33
                TL=TL.*sign(randn(size(TL)));
            case 46  % 14. random uniform numbers added for all loadings
                TL= TL2-.025+.05*rand(size(TL2));
            case 47
                TL= TL2-.05+.1*rand(size(TL2));
            case 48
                TL= TL2-.075+.15*rand(size(TL2));
            case 49
                TL= TL2-.10+.2*rand(size(TL2));
        end
        try
            [OBS,W]=Hong(m,p,N,TL,e,f,ifcw,eps,q);R=corrcoef(OBS);eigR=eig(R); eigR=sort(eigR);
            [Lobl,c,Fscores,Fscores_T,Rreduced]=FA(OBS,m,TL,R);
            
            eigRreduced=eig(Rreduced);eigRreduced=sort(eigRreduced);
            temp =diag(corr(Fscores,Fscores_T));
            FSC(REP,cases)=mean(temp); % calculate factor score correlation coefficient
            FSC1(REP,cases)=temp(1); % calculate factor score correlation coefficient
            FSC2(REP,cases)=temp(2); % calculate factor score correlation coefficient
            FSC3(REP,cases)=temp(3); % calculate factor score correlation coefficient
            CC(REP,cases)=cc(TL,Lobl); % calculate Tucker's congruence coefficient
            CC1(REP,cases)=cc(TL(:,1),Lobl(:,1)); % calculate Tucker's congruence coefficient
            CC2(REP,cases)=cc(TL(:,2),Lobl(:,2)); % calculate Tucker's congruence coefficient
            CC3(REP,cases)=cc(TL(:,3),Lobl(:,3)); % calculate Tucker's congruence coefficient
            HW(REP,cases)=max(c==1);  % see whether a unique variance is zero
            EF(REP,cases)=eigR(p-m+1);     % Eigenvalue m
            EFplus1(REP,cases)=eigR(p-m);  % Eigenvalue m+1
            pause
            EFreduced(REP,cases)=eigRreduced(p-m+1);
            EFreducedplus1(REP,cases)=eigRreduced(p-m);
        catch error
        end
        if mod(REP,100)==0
            [~,~,CICC]=ttest(CC(1:REP,cases));
        end
        if mod(REP,100)==0 && diff(CICC)<0.001
            break
        end
    end
end
%%
[MV,CR,CR1,CR2,CRreduced]=deal(NaN(1,cases));
for k=1:cases
    V1=EF(1:end,k)';
    V2=EFplus1(1:end,k)';
    V1reduced=EFreduced(1:end,k)';
    V2reduced=EFreducedplus1(1:end,k)';
    MV(k)=(mean(V1)*std(V2)+mean(V2)*std(V1))/(std(V1)+std(V2));
    CR1(k)=sum(V2>MV(k))./length(V1);
    CR2(k)=sum(V1<MV(k))./length(V1);
    CR(k)=cohend(V1,V2);
    CRreduced(k)=cohend(V1reduced,V2reduced);
end
disp([transpose(1:size(CC,2)) nanmean(CC)' prctile(CC,5)' nanmean(CC1)'  nanmean(CC2)' nanmean(CC3)' nanmean(HW)' nanmean(FSC)' nanmean(FSC1)' nanmean(FSC2)' nanmean(FSC3)' CR'])

M=size(CC,2);
figure;hold on;
temp=nanmean(CC(:,1));
plot(nanmean(CC1),'ko')
plot(nanmean(CC2),'ks')
plot(nanmean(CC3),'kd')
plot([0 M+1],[temp temp])
set(gca,'Xtick',[])
legend('Factor 1','Factor 2','Factor 3','location','southwest','autoupdate','off')
ylabel('Mean K', 'FontSize',20)
h = findobj(gcf,'FontName','Helvetica');
set(h,'FontSize',20)
for i=1.5:4:M
    plot([i i],[-.1 1.1],'color',[.5 .5 .5])
    text(i+2,0.99,num2str(1+round(i/4)),'FontSize',16,'HorizontalAlignment','Center')
end
set(gca,'Ylim',[0.75 1],'Xlim',[0 M+1])

figure;hold on
temp=nanmean(FSC(:,1));
plot(nanmean(FSC1),'ko')
plot(nanmean(FSC2),'ks')
plot(nanmean(FSC3),'kd')
plot([0 max(CASES)+1],[temp temp])
set(gca,'Xtick',[])
legend('Factor 1','Factor 2','Factor 3','location','southwest','autoupdate','off')
ylabel('Mean FSC','FontSize',20)
h = findobj(gcf,'FontName','Helvetica');set(h,'FontSize',20)
for i=1.5:4:M
    plot([i i],[-.1 1.1],'color',[.5 .5 .5])
    text(i+2,1.01,num2str(1+round(i/4)),'FontSize',16,'HorizontalAlignment','Center')
end
set(gca,'Xlim',[0 M+1],'Ylim',[0.85 1])

figure;hold on
temp=nanmean(HW(~isnan(CC(:,1)),1));
NMHW=NaN(size(HW,2),1);
for i=1:size(HW,2)
    NMHW(i)=nanmean(HW(~isnan(CC(:,i)),i));
end
plot(nanmean(HW),'ko')
plot([0 M+1],[temp temp])
set(gca,'Xtick',[])
ylabel('Heywood cases (proportion of solutions)', 'FontSize',20)
h = findobj(gcf,'FontName','Helvetica');set(h,'FontSize',20)
set(gca,'Ylim',[0 1])
for i=1.5:4:M
    plot([i i],[-.1 1.1],'color',[.5 .5 .5])
    text(i+2,0.85,num2str(1+round(i/4)),'FontSize',16,'HorizontalAlignment','Center')
end
set(gca,'Xlim',[0 M+1],'Ylim',[0 .85])

figure;hold on
temp=nanmean(EF(~isnan(CC(:,1)),1));
NMEF=NaN(size(EF,2),1);
for i=1:size(EF,2)
    NMEF(i)=nanmean(EF(~isnan(EF(:,i)),i));
end
plot(CR,'ko')
plot([0 M+1],[temp temp])
set(gca,'Xtick',[])
ylabel('ES','FontSize',20)
h = findobj(gcf,'FontName','Helvetica');set(h,'FontSize',20)
for i=1.5:4:M
    plot([i i],[-.1 15.1],'color',[.5 .5 .5])
    text(i+2,5.5,num2str(1+round(i/4)),'FontSize',16,'HorizontalAlignment','Center')
end
set(gca,'Xlim',[0 M+1],'Ylim',[0 5.5])

figure;hold on
temp=nanmean(EFreduced(~isnan(CC(:,1)),1));
NMEF=NaN(size(EF,2),1);
for i=1:size(EF,2)
    NMEF(i)=nanmean(EFreduced(~isnan(EFreduced(:,i)),i));
end
plot(CRreduced,'ko')
plot([0 M+1],[temp temp])
set(gca,'Xtick',[])
ylabel('ES (reduced)', 'FontSize',20)
h = findobj(gcf,'FontName','Helvetica');set(h,'FontSize',20)
for i=1.5:4:M
    plot([i i],[-.1 15.1],'color',[.5 .5 .5])
    text(i+2,5.5,num2str(1+round(i/4)),'FontSize',16,'HorizontalAlignment','Center')
end
set(gca,'Xlim',[0 M+1],'Ylim',[0 5.5])
