% De Winter, J. C. F., Dodou, D., & Wieringa, P. A. (2009). Exploratory factor analysis with small sample sizes. Multivariate Behavioral Research, 44, 147-181.

close all;clear all;clc %#ok<CLALL>

% X=csvread('gosling.csv',1); % Read file received from Gosling
% before=X(:,4);age=X(:,3);sex=X(:,2);language=X(:,1);
% X(X==0)=NaN;
% X=X(before==1 & age>=21 & age<=60 & sex>0 & language==2,:);clear temp; % include only those respondents who indicated that they never completed the BFI before, age between 21 and 60, gender filled in, language English
% X=X(~sum(isnan(X),2),:); % include only those respondents without missing values
% X=X(std(X,[],2)>0,:);    % include only those respondents who did not provide identical responses to all items
% age=X(:,3);sex=X(:,2);
% X=X(:,5:48);
% disp([mean(age) std(age) min(age) max(age)])
% disp([mean(sex)-1 std(sex) min(sex) max(sex)])

NF=5;
load('BFI.mat') % load X matrix with BFI data
disp(['Total sample size = ' num2str(length(X))])
NN=floor(logspace(log10(10),log10(200),25));
REPS=10000;
[K,CC,HW,RR,EF,EFplus1]=deal(NaN(length(NN),REPS,NF));
X2=X;
LL=size(X,1);
for m=1:NF
    F1=[1 6  11 16 21 26 31 36];         % extraversion
    F2=[2 7  12 17 22 27 32 37 42];      % agreeableness
    F3=[3 8  13 18 23 28 33 38 43];      % conscientiousness
    F4=[4 9  14 19 24 29 34 39];         % neuroticism
    F5=[5 10 15 20 25 30 35 40 41 44];   % openness
    switch m
        case 1
            temp=F1;
        case 2
            temp=[F1 F5];
        case 3
            temp=[F1 F5 F3];
        case 4
            temp=[F1 F5 F3 F4];
        case 5
            temp=[F1 F5 F3 F4 F2];
    end
    X=X2(:,temp);eR1=corrcoef(X);
    [Lobl_true,c_true,~,~,~,phi_true]=FA(X,m,rand(size(X,2),m),eR1);
    
    ALTRUE=abs(Lobl_true);
    disp(['Number of variables included = ' num2str(size(ALTRUE,1))]) % p
    disp('Primary loading, that is, the strongest loading per variable (mean, min, max of variables):')
    disp([round(mean(max(ALTRUE,[],2)),2) round(min(max(ALTRUE,[],2)),2) round(max(max(ALTRUE,[],2)),2)]) % Mean of primary loadings (MinMax)
    [~,b]=max(ALTRUE,[],2);
    ALTRUE2=ALTRUE;for i=1:size(ALTRUE,1);ALTRUE2(i,b(i))=NaN;end % for each variable, remove the highest loading
    disp('Secondary loading, that is, the second strongest loading per variable (mean, min, max of variables):')
    disp([round(mean(nanmax(ALTRUE2,[],2)),2) round(min(nanmax(ALTRUE2,[],2)),2) round(max(nanmax(ALTRUE2,[],2)),2)]) % Mean of secondary loadings (MinMax)
    disp('Communalities (mean, min, max):')
    disp([round(mean(c_true),2) round(min(c_true),2) round(max(c_true),2)]) % Mean communalities (MinMax)
    phi_true2=abs(phi_true);phi_true2(phi_true2>.9999)=NaN;
    disp('Interfactor correlation (mean, min, max of all interfactor correlations excluding 1s):')
    disp([round(mean(phi_true2(~isnan(phi_true2))),2) round(min(phi_true2(~isnan(phi_true2))),2) round(max(phi_true2(~isnan(phi_true2))),2)]) % Mean interfactor correlation (MinMax)
    for i2=1:length(NN)
        disp(i2)
        for i=1:REPS
            R=1;RR(i2,i,m)=R; % FLAG
            N=NN(i2);
            while R~=0 % do until FA is successfully conducted
                S=ceil(rand(ceil(N*1.5))*LL); S=unique(S);temp=randperm(length(S));temp=temp(1:N);S=S(temp); % draw a random sample with replacement
                try
                    eR2=corrcoef(X(S,:));eigR=sort(eig(eR2));
                    [Lobl,c,Fscores,Fscores_T]=FA(X(S,:),m,Lobl_true,eR2);
                    K(i2,i,m)=cc(Lobl,Lobl_true);
                    CC(i2,i,m)=mean(diag(corr(Fscores,Fscores_T)));
                    HW(i2,i,m)=max(c==1);
                    EF(i2,i,m)=eigR(size(eigR,1)-m+1);
                    EFplus1(i2,i,m)=eigR(size(eigR,1)-m);
                    R=0;
                catch
                    R=R+1;RR(i2,i,m)=R; % re-sampling
                end
            end
        end
    end
end
%%
CR=NaN(i2,m);
for k=1:m
    for kk=1:i2
        V1=EF(kk,1:i,k)';
        V2=EFplus1(kk,1:i,k)';
        CR(kk,k)=cohend(V1,V2);
    end
end

figure
for i3=1:m
    if i3==1
        semilogx(NN,nanmean(K(:,:,i3),2),'k');
    elseif i3==2
        semilogx(NN,nanmean(K(:,:,i3),2),'k:')
    elseif i3==3
        semilogx(NN,nanmean(K(:,:,i3),2),'k--')
    elseif i3==4
        semilogx(NN,nanmean(K(:,:,i3),2),'k-.')
    elseif i3==5
        semilogx(NN,nanmean(K(:,:,i3),2),'-','Color',[.5 .5 .5])
    end
    hold on
end
legend('E','E,O','E,O,C','E,O,C,N','E,O,C,N,A','location','southeast','autoupdate','off')
plot([min(NN) max(NN)],[.95 .95],'k')
h=findobj(gcf,'FontName','Helvetica');set(h,'FontSize',16)
xlabel('\it N', 'FontSize',20)
ylabel('Mean \it K','FontSize',20)
set(gca,'Xlim',[min(NN) max(NN)],'XTick',NN)

figure
for i3=1:m
    if i3==1
        semilogx(NN,nanmean(CC(:,:,i3),2),'k')
    elseif i3==2
        semilogx(NN,nanmean(CC(:,:,i3),2),'k:')
    elseif i3==3
        semilogx(NN,nanmean(CC(:,:,i3),2),'k--')
    elseif i3==4
        semilogx(NN,nanmean(CC(:,:,i3),2),'k-.')
    elseif i3==5
        semilogx(NN,nanmean(CC(:,:,i3),2),'-','Color',[.5 .5 .5])
    end
    hold on
end
legend('E','E,O','E,O,C','E,O,C,N','E,O,C,N,A','location','southeast','autoupdate','off')
plot([min(NN) max(NN)],[.95 .95],'k')
h = findobj(gcf,'FontName','Helvetica');set(h,'FontSize',16)
xlabel('\it N', 'FontSize',20)
ylabel('Mean \it FSC', 'FontSize',20)
set(gca,'Xlim',[min(NN) max(NN)],'XTick',NN)

figure
for i3=1:m
    if i3==1
        semilogx(NN,nanmean(HW(:,:,i3),2),'k')
    elseif i3==2
        semilogx(NN,nanmean(HW(:,:,i3),2),'k:')
    elseif i3==3
        semilogx(NN,nanmean(HW(:,:,i3),2),'k--')
    elseif i3==4
        semilogx(NN,nanmean(HW(:,:,i3),2),'k-.')
    elseif i3==5
        semilogx(NN,nanmean(HW(:,:,i3),2),'-','Color',[.5 .5 .5])
    end
    hold on
end
legend('E','E,O','E,O,C','E,O,C,N','E,O,C,N,A','location','northeast','autoupdate','off')
plot([min(NN) max(NN)],[.05 .05],'k')
h = findobj(gcf,'FontName','Helvetica');set(h,'FontSize',16)
xlabel('\it N', 'FontSize',20)
ylabel('Heywood cases (proportion of solutions)', 'FontSize',20)
set(gca,'Xlim',[min(NN) max(NN)],'XTick',NN)

figure
for i3=1:m
    if i3==1
        semilogx(NN,nanmean(CR(:,i3),2),'k');
    elseif i3==2
        semilogx(NN,nanmean(CR(:,i3),2),'k:')
    elseif i3==3
        semilogx(NN,nanmean(CR(:,i3),2),'k--')
    elseif i3==4
        semilogx(NN,nanmean(CR(:,i3),2),'k-.')
    elseif i3==5
        semilogx(NN,nanmean(CR(:,i3),2),'-','Color',[.5 .5 .5])
    end
    hold on
end
legend('E','E,O','E,O,C','E,O,C,N','E,O,C,N,A','location','northwest','autoupdate','off')
plot([min(NN) max(NN)],[4 4],'k')
h = findobj(gcf,'FontName','Helvetica');set(h,'FontSize',16)
xlabel('\it N', 'FontSize',20)
ylabel('ES','FontSize',20)
set(gca,'Xlim',[min(NN) max(NN)],'XTick',NN)

figure
for i3=1:m
    if i3==1
        semilogx(NN,-1+nanmean(RR(:,:,i3),2),'k')
    elseif i3==2
        semilogx(NN,-1+nanmean(RR(:,:,i3),2),'k:')
    elseif i3==3
        semilogx(NN,-1+nanmean(RR(:,:,i3),2),'k--')
    elseif i3==4
        semilogx(NN,-1+nanmean(RR(:,:,i3),2),'k-.')
    elseif i3==5
        semilogx(NN,-1+nanmean(RR(:,:,i3),2),'-','Color',[.5 .5 .5])
    end
    hold on
end
legend('E','E,O','E,O,C','E,O,C,N','E,O,C,N,A','location','southeast','autoupdate','off')
plot([min(NN) max(NN)],[.05 .05],'k')
h = findobj(gcf,'FontName','Helvetica');set(h,'FontSize',16)
xlabel('\it N', 'FontSize',20);ylabel('Number of re-samples','FontSize',20)
set(gca,'Xlim',[min(NN) max(NN)],'XTick',NN)