% De Winter, J. C. F., Dodou, D., & Wieringa, P. A. (2009). Exploratory factor analysis with small sample sizes. Multivariate Behavioral Research, 44, 147-181.

clear all;close all;clc;warning on all %#ok<CLALL>
ceilfix=@(x)ceil(abs(x)).*sign(x);
CB=combvec([.2 .4 .6 .8 .9],[1 2 3 4 8],[6 12 24 48 96])';
NT=[500 5000 10000]; % number of repetitions for the three phases

CC=zeros(size(CB,1),NT(3));
[NN,FSC,HW,NF,CF,FF]=deal(NaN(size(CB,1),NT(3)));
CR=NaN(1,size(CB,1));
for k=1:size(CB,1)
    NN(k,1)=1000; % initial N, N(1), is set at 1000
    lambda = CB(k,1); m = CB(k,2);p = CB(k,3);
    disp(CB(k,:))
    TL=zeros(p,m);
    FD=round(1:p/m:p+1); % equal as possible distribution of loading variables per factor
    for i=1:length(FD)-1
        TL(FD(i):FD(i+1)-1,i)=lambda; % true factor loading matrix
    end
    if p>=m*2 % only perform FA when the number of variables is at least twice the number of factors
        for j=1:NT(3)
            FLAG=1;FF(k,j)=FLAG;
            N=NN(k,j);
            while FLAG~=0 % do until FA is successfully conducted
                try               
                    X=Hong(m,p,N,TL,0,0,1,0,0);
                    R=corrcoef(X); % generate observation matrix using Hong's method
                    [Lobl,c,Fscores,Fscores_T,RR]=FA(X,m,TL,R);  % perform PAF, extracting m factors
                    CC(k,j)=cc(TL,Lobl);          % Tucker's congruence coefficient
                    FLAG=0;
                catch error
                    FLAG=FLAG+1;
                    FF(k,j)=FLAG;
                end
            end
            if NN(k,j)>10000;break;end    % exit when N rises above 10000
            if j<NT(2) % controlling phase
                NN(k,j+1)=NN(k,j)-ceilfix(NN(k,j)*(CC(k,j)-.95));  % calculate N for next step
            elseif j==NT(2) % deciding upon N
                NN(k,j+1)=round(nanmean(NN(k,NT(1)+1:NT(2)))); % mean N for reps 501 to 5000
            else  % constant N phase
                NN(k,j+1)=NN(k,j); % keep N constant
                eigr=eig(R);eigr=sort(eigr);
                FSC(k,j)=mean(diag(corr(Fscores,Fscores_T))); % Factor score correlation coefficient
                HW(k,j)=sign(sum(c==1));                      % Heywood case (1) or not (0)
                NF(k,j)=eigr(p-m+1);                          % Eigenvalue m
                CF(k,j)=eigr(p-m);                            % Eigenvalue m+1
            end
            if NN(k,j+1)<5;NN(k,j+1)=5;end  % minimum N
        end
    end
    CR(k)=abs(cohend(CF(k,NT(2)+1:end),NF(k,NT(2)+1:end)));
end
%%
disp([CB mean(NN(:,NT(2)+1:NT(3)),2) median(CC(:,NT(2)+1:NT(3)),2) mean(CC(:,NT(2)+1:NT(3)),2) prctile(CC(:,NT(2)+1:NT(3))',5)' mean(FSC(:,NT(2)+1:NT(3)),2) mean(HW(:,NT(2)+1:NT(3)),2) CR'])
